/* $Id: katerubyplugin.cpp,v 1.4 2002/02/25 11:21:41 pi Exp $
 *
 * Copyright (C) 2001 Pirmin Kalberer
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#include <kdebug.h>
#include "katerubyplugin.h"
#include <kaction.h>
#include <kinstance.h>
#include <kiconloader.h>
#include <qmessagebox.h>
#include <klocale.h>

#include <stdlib.h>

//-------------------

/** View for Interpreter output */
static Kate::View* outview = 0;

EXTERN VALUE rb_defout;

  /** Called by Ruby for writing to STDOUT. Writes into window */
static VALUE defout_write(VALUE self, VALUE str)
{
	outview->insertText(KateRubyPlugin::val2str(str));
    return Qnil;
}

//-------------------

/** Get ruby error/exception info an print it */
void KateRubyPlugin::exception_print(Kate::View* errview)
{
	//Adapted from eruby_main by Shugo Maeda <shugo@modruby.net>
	VALUE errat;
	VALUE eclass;
	VALUE einfo;

	if (NIL_P(ruby_errinfo)) return;

	errat = rb_funcall(ruby_errinfo, rb_intern("backtrace"), 0);
	if (!NIL_P(errat)) {
		VALUE mesg = RARRAY(errat)->ptr[0];

		if (NIL_P(mesg)) {
	  	ID last_func = rb_frame_last_func();
 	  	errview->insertText(rb_id2name(last_func));
		}	else {
			errview->insertText(QString::fromLocal8Bit(RSTRING(mesg)->ptr, RSTRING(mesg)->len));
		}
  }

  eclass = CLASS_OF(ruby_errinfo);
  einfo = rb_obj_as_string(ruby_errinfo);
  if (eclass == rb_eRuntimeError && RSTRING(einfo)->len == 0) {
		errview->insertText(": unhandled exception\n");
  } else {
		VALUE epath;

		epath = rb_class_path(eclass);
		if (RSTRING(einfo)->len == 0) {
	    errview->insertText(": ");
			errview->insertText(QString::fromLocal8Bit(RSTRING(epath)->ptr, RSTRING(epath)->len));
			errview->insertText("\n");
		}	else {
	    char *tail  = 0;
	    int len = RSTRING(einfo)->len;

	    if (RSTRING(epath)->ptr[0] == '#') epath = 0;
	    if ((tail = strchr(RSTRING(einfo)->ptr, '\n')) != NULL) {
				len = tail - RSTRING(einfo)->ptr;
				tail++;		/* skip newline */
	    }
	    errview->insertText(": ");
			errview->insertText(QString::fromLocal8Bit(RSTRING(einfo)->ptr, len));
	    if (epath) {
		    errview->insertText(" (");
		    errview->insertText(QString::fromLocal8Bit(RSTRING(epath)->ptr, RSTRING(epath)->len));
		    errview->insertText(")\n");
	    }
	    if (tail) {
		    errview->insertText(QString::fromLocal8Bit(tail, RSTRING(einfo)->len - len - 1));
		    errview->insertText("\n");
	    }
		}
  }

  if (!NIL_P(errat)) {
		int i;
		struct RArray *ep = RARRAY(errat);

#define TRACE_MAX (TRACE_HEAD+TRACE_TAIL+5)
#define TRACE_HEAD 8
#define TRACE_TAIL 5

		rb_ary_pop(errat);
		ep = RARRAY(errat);
		for (i=1; i<ep->len; i++) {
	    if (TYPE(ep->ptr[i]) == T_STRING) {
		    errview->insertText("        from ");
		    errview->insertText(QString::fromLocal8Bit(RSTRING(ep->ptr[i])->ptr,
					RSTRING(ep->ptr[i])->len));
		    errview->insertText("\n");
	    }
	    if (i == TRACE_HEAD && ep->len > TRACE_MAX) {
		    errview->insertText(QString("         ... %ld levels...<br></div>\n")
					.arg(ep->len - TRACE_HEAD - TRACE_TAIL));
				i = ep->len - TRACE_TAIL;
	    }
		}
  }
}

/** Run Ruby interpreter with @p filename */
void KateRubyPlugin::run_file(Kate::View* outview, const char* filename){
  int status;
	//ruby_options(intargc, char**argv")
	rb_load_protect(rb_str_new2(filename), 0, &status);
	outview->bottom();
	if (status) {
		exception_print(outview);
	}
	outview->insertText("\n");
}
/** Do evaluation */
void KateRubyPlugin::eval(Kate::View* outview, const char* code){
  int status;
	QString ret_str;
	/*VALUE ret =*/ rb_eval_string_protect(code, &status);
	outview->bottom();
	if (status) {
		exception_print(outview);
	} else {
		//outview->insertText(QString("\nRetval: ")+val2str(ret));
	}
	outview->insertText("\n");
}

/** Convert Ruby value to a string */
QString KateRubyPlugin::val2str(const VALUE rval){
	QString ret_str;
  switch (TYPE(rval)) {
		case T_STRING: return STR2CSTR(rval);
  	case T_NIL   : return "(nil)";
  	case T_FIXNUM: return ret_str.setNum(FIX2LONG(rval));
  	case T_FLOAT : return ret_str.setNum(NUM2DBL(rval));
  	case T_OBJECT:
  	case T_CLASS :
  	case T_ICLASS:
  	case T_MODULE:
  	case T_REGEXP:
  	case T_ARRAY :
  	case T_HASH  :
  	case T_STRUCT:
  	case T_BIGNUM:
  	case T_FILE  :
		default:
			ret_str = "(unsupported type)";
   }
	return ret_str;
}

KateRubyPlugin::KateRubyPlugin( QObject* parent, const char* name )
    : Kate::Plugin( parent, name )
{
	ruby_init();
	ruby_init_loadpath();
	rb_set_safe_level(0); //FIXME
	ruby_script("ruby");

  rb_defout = rb_str_new("", 0);
  rb_define_singleton_method(rb_defout, "write", (VALUE(*)())defout_write, 1);

	//found no other way to get a reopenable view :-(
	msgfileURL = new KURL(tmpnam(0));
}
 
KateRubyPlugin::~KateRubyPlugin()
{
}
 
Kate::PluginView *KateRubyPlugin::createView (Kate::MainWindow *win)
{
   Kate::PluginView *view = new Kate::PluginView (this, win);

  (void) new KAction ( i18n("Run"), Key_F9, this,
                      SLOT( run() ), view->actionCollection(),
                      "kruby_run" );
  (new KAction ( i18n("Run..."), 0, this,
                      SLOT( run() ), view->actionCollection(),
                      "kruby_run_with_args" ))->setEnabled(false);
  (void) new KAction ( i18n("Evaluate Selection"), CTRL+Key_E, this,
                      SLOT( eval_selection() ), view->actionCollection(),
                      "kruby_eval" );

   view->setXML( "plugins/kateruby/ui.rc" );

   return view;
}

/** Run Ruby interpreter with active view */
void KateRubyPlugin::run(){
  Kate::View *kv = myApp->getViewManager()->getActiveView();

  if (!kv) return;

	Kate::Document* srcdoc = kv->getDoc();
	if (!srcdoc->save()) return;

	KURL srcurl = srcdoc->url();

  myApp->getViewManager()->openURL( *msgfileURL );
  outview = myApp->getViewManager()->getActiveView();

  run_file(outview, srcurl.path());

  myApp->getViewManager()->openURL(srcurl);
}

void KateRubyPlugin::eval_selection()
{
  Kate::View *kv = myApp->getViewManager()->getActiveView();

  if (!kv) return;

	QString markedText = kv->markedText();
	if (markedText.length() == 0) return;

	Kate::Document* srcdoc = kv->getDoc();
	KURL srcurl = srcdoc->url();

  myApp->getViewManager()->openURL( *msgfileURL );
  outview = myApp->getViewManager()->getActiveView();

  eval(outview, markedText);

	if (!srcurl.isEmpty())
		myApp->getViewManager()->openURL(srcurl);
}

//-------------------

KPluginFactory::KPluginFactory( QObject* parent, const char* name )
  : KLibFactory( parent, name )
{
  s_instance = new KInstance("KPluginFactory");
}
 
QObject* KPluginFactory::createObject( QObject* parent, const char* name, const char*, const QStringList & )
{
    QObject *obj = new KateRubyPlugin( parent, name );
    return obj;
}
KPluginFactory::~KPluginFactory()
{ delete s_instance; }

 
extern "C"
{
  void* init_libkaterubyplugin()
  {
    return new KPluginFactory;
  }
 
}
 
KInstance* KPluginFactory::s_instance = 0L;
